@@ -1,24 +1,15 @@
-/* $Id: buffer.c,v 1.10 2002/06/28 11:27:53 indoh Exp $ */
-/* command/buffer.c */
-
 /************************************************************
  *
- * COPYRIGHT (C) HITACHI,LTD. 2002 ALL RIGHTS RESERVED.
+ * COPYRIGHT (C) HITACHI,LTD. 2002-2003 ALL RIGHTS RESERVED.
  * WRITTEN BY HITACHI SYSTEMS DEVELOPMENT LABORATORY,
  *            HITACHI CENTRAL RESEARCH LABORATORY.
  *
  * Created by K.Hatasaki <keisukeh@crl.hitachi.co.jp>
+ * Updated by M.Hiramatsu <hiramatu@sdl.hitachi.co.jp>
  *
  ************************************************************/
 
-#include <time.h>		/* for struct timeval */
-#include "command.h"
-
-#define __KERNEL__
-#include <linux/wait.h>
-#undef __KERNEL__
-#define CMD_LKSTDUMP
-#include <linux/lkst_private.h>
+#include "buffer.h"
 
 #define LKST_ETYPE_DEF_DESCRIPTIONS
 #include <linux/lkst_events.h>
@@ -26,161 +17,423 @@
 #define LKST_ETYPE_DEF_DATATABLE
 #include <linux/lkst_events.h>
 
-#define _LINUX_BUFFER_H
-#include "buffer.h"
+#define UNDEF_STR "undefined"
 
-inline void buffer_list(void)
+int lkst_get_status(struct lkst_status_param *sparam)
+{
+	int retval,fd;
+	fd = open_dev();
+	retval = ioctl(fd, LKST_IOC_TRC_STATUS, sparam); /* trc_status */
+	close(fd);
+	return retval;
+}
+
+/* display as classical format */
+void classical_buffer_list(void)
 {
 	int retval;
 	int fd;
-	int i;
-	char prev_id_str[4], next_id_str[4];
-	char flag[2] = {' ','*'};
+	int i,j,k;
+	char prev_id_str[5]={0}, next_id_str[5]={0};
 
 	lkst_buffer_id buffer_id;
 	struct lkst_buffer_listparam lp;
 	struct lkst_buffer_listent listent[LKST_BUFFER_TBL_MAX];
+	struct lkst_status_param sparam;
 
 	fd = open_dev();
 
-	lp.listent_size = LKST_BUFFER_LISTENT_SIZE(LKST_BUFFER_TBL_MAX);
-	lp.listent = listent;
-
-	retval = ioctl(fd, LKST_IOC_BUFFER_LIST, &lp);	/* buffer list */
-	
-	if (retval) {
-		fprintf(stderr, "ioctl() error: %s\n", strerror(errno));
+	retval = ioctl(fd, LKST_IOC_TRC_STATUS, &sparam); /* trc_status */
+	if (retval<0) {
+		fprintf(stderr, "ioctl(LKST_IOC_TRC_STATUS) error: %s\n", strerror(errno));
 		exit(1);
 	}
+	printf(" id\t cpu\tsize\tprev_id\tnext_id\t    readp\t    writep\twrap\n");	
+	printf("%3d\t%3d%c%9d\t%3s\t%3s\t%9d\t%9d\t %1d\n",
+	       255, 0, ' ', 
+	       sparam.static_buffer_size,
+	       "NUL", "NUL", 0,
+	       sparam.static_buffer_recid % sparam.static_buffer_size,
+	       0);
 
-	printf(" id\t cpu\tsize\tprev_id\tnext_id\t    readp\t    writep\twrap\n");
-	for (i = 0; i < LKST_BUFFER_TBL_MAX; i++) {
-		if (lp.listent[i].id != LKST_BUFFER_ID_VOID) {
-		  	if (lp.listent[i].prev == LKST_BUFFER_ID_VOID)
+
+	for_each_online_cpus( j, sparam.online_cpu) {
+		lp.cpu = j;
+		lp.listent_num = LKST_BUFFER_TBL_MAX;
+		lp.listent = listent;
+		retval = ioctl(fd, LKST_IOC_BUFFER_LIST, &lp);	/* buffer list */
+		if (retval<0) {
+			fprintf(stderr, "ioctl(LKST_IOC_BUFFER_LIST) error: %s\n", strerror(errno));
+			exit(1);
+		}
+		if (lp.listent_num < lp.buffer_num) lp.buffer_num = lp.listent_num;
+		for (i = 0; i < lp.buffer_num; i++) {
+			for (k=0;k<lp.buffer_num && listent[k].next!=listent[i].id;k++);
+		  	if (k == lp.buffer_num)
 				strncpy (prev_id_str, "NUL", 4);
 			else
-				sprintf (prev_id_str, "%3d", lp.listent[i].prev);
+				snprintf (prev_id_str, 4, "%3d", listent[k].id);
 		  	if (lp.listent[i].next == LKST_BUFFER_ID_VOID)
 				strncpy (next_id_str, "NUL", 4);
 			else
-				sprintf (next_id_str, "%3d", lp.listent[i].next);
+				snprintf (next_id_str, 4, "%3d", lp.listent[i].next);
 	  		printf("%3d\t%3d%c%9d\t%3s\t%3s\t%9d\t%9d\t %1d\n",
        			       lp.listent[i].id,
-			       lp.listent[i].cpu,
-			       flag[lp.listent[i].current_flag],
+			       lp.cpu,
+			       (lp.listent[i].id==lp.write_buf)?'*':' ',
 			       lp.listent[i].size,
 			       prev_id_str,
 			       next_id_str,
-			       lp.listent[i].readp,
-			       lp.listent[i].writep,
-			       lp.listent[i].wrap_flag);
+			       ((lp.listent[i].id==lp.read_buf)?lp.read_pos:LKST_BUFFER_INIT_POS)*64,
+			       (listent[i].baseid - listent[i].write_offset)*LKST_EVENT_RECORD_BYTE,
+			       0);
 		}
 	}
-
 	close(fd);
 }
 
-inline void buffer_create(int cpu, lkst_buffer_id id, size_t size)
+static inline char * itosn(char *str, int i, size_t n)
+{
+	snprintf(str, n, "%d", i);
+	return str;
+}
+
+static char * bid2str(char *strbuf, lkst_buffer_id id)
+{
+	if (id == LKST_BUFFER_ID_VOID) {
+		strncpy(strbuf," --", 4);
+	}else
+		snprintf(strbuf, 4, "%3d",id);
+	return strbuf;
+}
+
+static int lkst_get_cpu_online_map(int fd) 
+{
+	int retval;
+	struct lkst_status_param param;
+
+	retval = ioctl(fd, LKST_IOC_TRC_STATUS, &param); /* trc_status */
+	if (retval<0) {
+		fprintf(stderr, "ioctl(LKST_IOC_TRC_STATUS) error: %s\n", strerror(errno));
+		exit(1);
+	}
+	return param.online_cpu;
+}
+
+static void lkst_buffer_static_list(FILE *fout, int fd, 
+				    struct lkst_status_param *param)
+{
+	int retval;
+	retval = ioctl(fd, LKST_IOC_TRC_STATUS, param); /* trc_status */
+	if (retval<0) {
+		fprintf(stderr, "ioctl(LKST_IOC_TRC_STATUS) error: %s\n", strerror(errno));
+		exit(1);
+	}
+
+	fprintf(fout, "Static buffer Size: %9d Recid: %9d\n",
+		param->static_buffer_size, param->static_buffer_recid);
+}
+
+static int lkst_buffer_list(FILE *fout, int fd, int cpu, int level)
+{
+	int retval;
+	int i;
+	char buf[4][8];
+	lkst_buffer_id buffer_id;
+	struct lkst_buffer_listparam lp={0};
+	struct lkst_buffer_listent listent[LKST_BUFFER_TBL_MAX]={0};
+
+	lp.cpu = cpu;
+	lp.listent_num = LKST_BUFFER_TBL_MAX;
+	lp.listent = listent;
+
+	if ( ioctl(fd, LKST_IOC_BUFFER_LIST, &lp) < 0)
+		return -1;
+
+	fprintf(fout,"CPU:%2d  Buffers: %3d  Owner_pid:%5s\n",
+		lp.cpu, lp.buffer_num,
+		(lp.owner==PID_MAX+1)?"--":itosn(buf[0],lp.owner,6));
+	
+	if (lp.listent_num < lp.buffer_num) lp.buffer_num = lp.listent_num;
+   
+	fprintf(fout, "cpu  id   readp  writep    size size[Byte]   recid   R W  next rq_p rq_n \n");
+	for (i = 0; i < lp.buffer_num; i++) {
+		fprintf(fout," %2d %3d %7s %7d %7d %9d %8x   %c %c   %3s %4s %4s\n",
+			lp.cpu,
+			listent[i].id,
+			(listent[i].id==lp.read_buf)?
+				itosn(buf[3],lp.read_pos,8):"--",
+			listent[i].baseid - listent[i].write_offset,
+			listent[i].size/LKST_EVENT_RECORD_BYTE,
+			listent[i].size,
+			LKST_BUFFER_RECID(listent[i].id, listent[i].baseid),
+			(listent[i].id==lp.read_buf)?'<':'-',
+			(listent[i].id==lp.write_buf)?'>':'-',
+			bid2str(buf[0],listent[i].next),
+			(listent[i].id==lp.rq_head)?
+				"head":bid2str(buf[1],listent[i].rq_prev),
+			(listent[i].id==lp.rq_tail)?
+				"tail":bid2str(buf[2],listent[i].rq_next));
+	}
+	return 0;
+}
+
+
+void buffer_list(int cpu_id, int level)
+{
+	int cpu;
+	int fd;
+	struct lkst_status_param param={0};
+	fd = open_dev();
+	if(fd <0) {
+		fprintf(stderr,"open() error: %s\n", strerror(errno));
+		exit(1);
+	}
+	if(cpu_id == -1) {
+		lkst_buffer_static_list(stdout,fd,&param);
+	}
+	for(cpu = 0;cpu<LKST_CPU_MAX;cpu++) {
+		if( cpu != cpu_id ){
+			if (cpu_id != -1 || 
+			    !(param.online_cpu & (1UL<<cpu))) continue;
+		}
+		
+		if ( lkst_buffer_list(stdout, fd, cpu, level) < 0) {
+			fprintf(stderr, "ioctl(LKST_IOC_BUFFER_LIST) error: %s\n", strerror(errno));
+			continue;
+		}
+	}
+	close(fd);
+}
+
+void buffer_link(int cpu, lkst_buffer_id id,lkst_buffer_id next)
+{
+	struct lkst_buffer_linkparam param;
+	int fd,retval;
+
+	fd = open_dev();
+
+	if ( cpu != -1 ) {
+		param.cpu = cpu;		
+		param.id = id ;
+		param.next = next;
+		retval = ioctl(fd, LKST_IOC_BUFFER_LINK, &param);
+		if (retval < 0) {
+			fprintf(stderr, "ioctl(LKST_IOC_BUFFER_LINK) error: %s\n", strerror(errno));
+			exit(1);
+		} else {
+			if (next != LKST_BUFFER_ID_VOID) {
+				printf("Buffer ID %d is linked to ID %d on CPU %d\n", id, next, cpu);
+			}else { 
+				printf("Link from buffer ID %d is disconnected on CPU %d\n", id, cpu);
+			}
+		}
+	}else {
+		int online;
+		online = lkst_get_cpu_online_map(fd);
+		for_each_online_cpus(cpu, online) {
+			param.cpu = cpu;		
+			param.id = id ;
+			param.next = next;
+			retval = ioctl(fd, LKST_IOC_BUFFER_LINK, &param);
+			if (retval < 0) {
+				fprintf(stderr, "ioctl(LKST_IOC_BUFFER_LINK) error: %s\n", strerror(errno));
+			} else {
+				if (next != LKST_BUFFER_ID_VOID) {
+					printf("Buffer ID %d is linked to ID %d on CPU %d\n", id, next, cpu);
+				}else { 
+					printf("Link from buffer ID %d is disconnected on CPU %d\n", id, cpu);
+				}
+			}
+		}
+	}
+	
+	close(fd);
+}
+
+void buffer_create(int cpu, lkst_buffer_id id, 
+		   size_t size, lkst_buffer_id nxid)
 {
 	int retval;
 	int fd;
 	struct lkst_buffer_param param;
 
 	fd = open_dev();
-
-	param.cpu = cpu;
-	param.id = id;
-	param.size = size;
-
-	retval = ioctl(fd, LKST_IOC_BUFFER_CREATE, &param); /* buffer_create */
-
-	if (retval) {
-		fprintf(stderr, "ioctl() error: %s\n", strerror(errno));
-		exit(1);
-	}
-
-	if (cpu != -1)
-		printf("New buffer was created, id=%d size=%d\n",
-	       		param.id, param.result_size);
-	else
-		printf("New buffers were created for all CPUs, size=%d\n",
-			param.result_size);
-
-	close(fd);
-}
-
-inline void buffer_delete(lkst_buffer_id id)
-{
-	int retval;
-	int fd;
-
-	fd = open_dev();
-
-	retval = ioctl(fd, LKST_IOC_BUFFER_DELETE, id);	/* buffer_delete */
-
-	if (retval) {
-		fprintf(stderr, "ioctl() error: %s\n", strerror(errno));
-		exit(1);
-	}
-
-	printf("Buffer id=%d was deleted.\n", id);
-
-	close(fd);
-}
-
-inline void buffer_shift(int cpu)
-{
-	int fd;
-	int retval;
-
-	fd = open_dev();
-
-	retval = ioctl(fd, LKST_IOC_BUFFER_SHIFT, cpu);	/* buffer_shift */
-
-	if (retval) {
-		fprintf(stderr, "ioctl() error: %s\n", strerror(errno));
-		exit(1);
-	}
 	
-	printf("Currently selected buffer changed on cpu=%d\n", cpu);
+	if ( cpu != -1 ) {
+		param.cpu = cpu;
+		param.id = id;
+		param.size = size;
+		param.next = nxid;
 
+		retval = ioctl(fd, LKST_IOC_BUFFER_CREATE, &param); /* buffer_create */
+		
+		if (retval < 0) {
+			fprintf(stderr, "ioctl(LKST_IOC_BUFFER_CREATE) error: %s\n", strerror(errno));
+			exit(1);
+		}
+		printf("New buffer was created, cpu=%d, id=%d size=%d + %d(margin)\n",
+		       param.cpu, param.id, param.result_size-LKST_BUFFER_MARGIN_BYTE,
+		       LKST_BUFFER_MARGIN_BYTE );
+	}else {
+		int online;
+		online = lkst_get_cpu_online_map(fd);
+		for_each_online_cpus(cpu, online) {
+			param.cpu = cpu;
+			param.id = id;
+			param.size = size;
+			param.next = nxid;
+			
+			retval = ioctl(fd, LKST_IOC_BUFFER_CREATE, &param); /* buffer_create */
+			
+			if (retval < 0) {
+				fprintf(stderr, "ioctl(LKST_IOC_BUFFER_CREATE) error: %s\n", strerror(errno));
+				fprintf(stderr, "Failed to create a buffer on CPU %d\n",cpu);
+			}else
+				printf("New buffer was created, cpu=%d, id=%d size=%d + %d(margin)\n",
+				       param.cpu, param.id, param.result_size-LKST_BUFFER_MARGIN_BYTE,
+				       LKST_BUFFER_MARGIN_BYTE );
+		}
+	}
 	close(fd);
 }
 
-inline void buffer_shift_all(void)
+
+void buffer_delete(int cpu, lkst_buffer_id id)
 {
 	int retval;
 	int fd;
-	int i;
-	struct lkst_status_param param;
+	struct lkst_buffer_delparam param;
 
 	fd = open_dev();
 
-	retval = ioctl(fd, LKST_IOC_TRC_STATUS, &param); /* trc_status */
-
-	if (retval) {
-		fprintf(stderr, "ioctl() error: %s\n", strerror(errno));
-		exit(1);
+	if ( cpu != -1 ) {
+		param.cpu = cpu;
+		param.id = id;
+		retval = ioctl(fd, LKST_IOC_BUFFER_DELETE, &param);	/* buffer_delete */
+		
+		if (retval < 0) {
+			fprintf(stderr, "ioctl(LKST_IOC_BUFFER_DELETE) error: %s\n", strerror(errno));
+			exit(1);
+		}
+		printf("Buffer id=%d was deleted.\n", id);
+	}else{
+		int online;
+		online = lkst_get_cpu_online_map(fd);
+		for_each_online_cpus(cpu, online) {
+			param.cpu = cpu;
+			param.id = id;
+			retval = ioctl(fd, LKST_IOC_BUFFER_DELETE, &param);	/* buffer_delete */
+			
+			if (retval < 0) {
+				fprintf(stderr, "ioctl(LKST_IOC_BUFFER_DELETE) error: %s\n", strerror(errno));
+				fprintf(stderr, "Failed to delete a buffer on CPU %d\n",cpu);
+			}else
+				printf("Buffer id=%d was deleted.\n", id);
+		}
 	}
+	close(fd);
+}
 
-	for(i = 0; i < LKST_CPU_MAX; i++) {
-		if (param.current_buffer_id[i] != LKST_BUFFER_ID_VOID) {
-		  retval = ioctl(fd, LKST_IOC_BUFFER_SHIFT, i);
-							/* buffer_shift */
+int lkst_buffer_jump(int fd, int cpu, lkst_buffer_id dest)
+{
+	struct lkst_buffer_jumpparam param;
+	param.cpu = cpu;
+	param.dest = dest;
+	return ioctl(fd, LKST_IOC_BUFFER_JUMP, &param);
+}
 
-		  if (retval)
-			  fprintf(stderr, "Error: cannot change buffer of cpu=%d\n", i);
-		  else
-			  printf("Currently selected buffer changed on cpu=%d\n", i);
+int lkst_buffer_shift(int fd, int cpu)
+{
+	return ioctl(fd, LKST_IOC_BUFFER_SHIFT, cpu);
+}
 
+void buffer_shift(int cpu)
+{
+	int fd;
+	int retval;
+
+	fd = open_dev();
+
+	if ( cpu != -1 ) {
+		retval = lkst_buffer_shift(fd, cpu);	/* buffer_shift */
+		if (retval < 0) {
+			fprintf(stderr, "lkst_buffer_shift() error: %s\n", strerror(errno));
+			exit(1);
+		}
+		printf("Currently selected buffer changed on CPU %d\n", cpu);
+	}else {
+		int online;
+		online = lkst_get_cpu_online_map(fd);
+		for_each_online_cpus(cpu, online) {
+			retval = lkst_buffer_shift(fd, cpu);	/* buffer_shift */
+			if (retval < 0) {
+				fprintf(stderr, "lkst_buffer_shift() error: %s\n", strerror(errno));
+				fprintf(stderr, "Failed to change current buffer on CPU %d\n",cpu);
+			}else
+				printf("Currently selected buffer changed on CPU %d\n", cpu);
 		}
 	}
 
 	close(fd);
 }
 
-inline void buffer_read_write_to_file(int fd_o,
-				      struct lkst_log_buffer *lbuffer)
+void buffer_jump(int cpu, lkst_buffer_id dest)
+{
+	int fd;
+	int retval;
+
+	fd = open_dev();
+
+	if ( cpu != -1 ) {
+		retval = lkst_buffer_jump(fd, cpu, dest);	/* buffer_shift */
+		if (retval < 0) {
+			fprintf(stderr, "lkst_buffer_jump() error: %s\n", strerror(errno));
+			exit(1);
+		}
+		printf("Currently selected buffer changed to %d on CPU %d\n", dest, cpu);
+	} else {
+		int online;
+		online = lkst_get_cpu_online_map(fd);
+		for_each_online_cpus(cpu, online) {
+			retval = lkst_buffer_jump(fd, cpu, dest);	/* buffer_shift */
+			if (retval < 0) {
+				fprintf(stderr, "lkst_buffer_jump() error: %s\n", strerror(errno));
+				fprintf(stderr, "Failed to change current buffer on CPU %d\n",cpu);
+			}else
+				printf("Currently selected buffer changed to %d on CPU %d\n", dest, cpu);
+		}
+	}
+	close(fd);
+}
+
+
+int lkst_set_rmod(int fd, int cpu, struct lkst_log_buffer *lbuffer)
+{
+	struct lkst_buffer_srmodparam param;
+	int ret;
+	param.cpu = cpu;
+	param.mode = STD;
+	ret = ioctl(fd, LKST_IOC_BUFFER_SETRMOD, &param);
+	if( ret >= 0) {
+		lbuffer->xtime = param.xtime;
+		lbuffer->tsc = param.tsc;
+		lbuffer->cpu_freq = param.cpu_freq;
+		lbuffer->endian_big = param.endian_big;
+		lbuffer->buf_ver = param.buf_ver;
+		strncpy(lbuffer->arch, param.arch,4);
+	}
+	return ret;
+}
+
+int lkst_buffer_read(int fd, struct lkst_log_buffer *lbuffer)
+{
+	int ret;
+	return ioctl(fd, LKST_IOC_BUFFER_READ, lbuffer);
+}
+
+void buffer_write_to_file(int fd_o,
+			  struct lkst_log_buffer *lbuffer)
 {
 	int j;
 	u_int64_t cpu_hz; 
@@ -257,7 +510,7 @@ inline void buffer_read_write_to_file(int fd_o,
 	write(fd_o, lbuffer->buffer, lbuffer->result_read_size);
 }
 
-inline void buffer_read_display(struct lkst_log_buffer *lbuffer)
+void buffer_display(struct lkst_log_buffer *lbuffer)
 {	
 	int i, read_count;
 	struct lkst_log_record *buffer;
@@ -271,7 +524,7 @@ inline void buffer_read_display(struct lkst_log_buffer *lbuffer)
 	read_count =
 	    lbuffer->result_read_size / sizeof(struct lkst_log_record);
 
-	printf("result_read_size=%d, mum_of_read_entry=%d\n",
+	printf("result_read_size=%d, num_of_read_entry=%d\n",
 	       lbuffer->result_read_size, read_count);
 
 	printf("time: %09d [sec], %09d [usec], %018llu [mc]; cpu_freq: %09d [kHz]\n\n",
@@ -300,139 +553,118 @@ inline void buffer_read_display(struct lkst_log_buffer *lbuffer)
 	}
 }
 
-inline void buffer_read(lkst_buffer_id id, unsigned int num, char *file_name, 
-		 int num_set, int file_set)
+void buffer_read_all(int cpu, unsigned int num, int fout)
 {
-	int retval;
-	int fd, fd_o;
-	int i, i_max = 1;
-	int cnt = 0;
-	struct lkst_log_record *buffer;
-	struct lkst_log_buffer lbuffer;
-	struct lkst_buffer_listparam lp;
-	struct lkst_buffer_listent listent[LKST_BUFFER_TBL_MAX];
-
-	fd = open_dev();
-
-	if (file_set) {
-		if (0 == strncmp("-", file_name, 2))
-			fd_o = 1;
-		else
-			if ((fd_o = open(file_name, O_WRONLY|O_CREAT|O_TRUNC,0644)) < 0) {
-				perror("output");
-				exit(1);
-			}
-	}
-
-	if (id == LKST_BUFFER_ID_VOID || !num_set) {
-				/* buffer list */
-		lp.listent_size = LKST_BUFFER_LISTENT_SIZE(LKST_BUFFER_TBL_MAX);
-		lp.listent = listent;
-
-		if (retval = ioctl(fd, LKST_IOC_BUFFER_LIST, &lp)) {
-			fprintf(stderr, "ioctl() error: %s\n", strerror(errno));
-			perror("buffer_list");
-			exit(1);
-		}
-
-		i_max = LKST_BUFFER_ID_MAX;
-	}
-
-	for (i = 0; i <= i_max; i++) {
-
-		if (i_max == 1)
-			lbuffer.id = id;
-		else {
-			lbuffer.id = lp.listent[i].id;
-
-			if (lbuffer.id == LKST_BUFFER_ID_VOID)
-				continue;
-
-			if (id != LKST_BUFFER_ID_VOID && lbuffer.id != id)
-				continue;
-		}
-
-		cnt++;
-
-				/* read and write log buffer */
-		lbuffer.read_size = num_set ?
-			sizeof(struct lkst_log_record) * num : 
-			(lp.listent[i].size *
-			 sizeof(struct lkst_log_record) /
-			 LKST_SIZEOF_LKST_EVENT_RECORD);
-
-		if (NULL == (buffer = (struct lkst_log_record *) malloc(lbuffer.read_size))) {
-			perror("malloc");
-			exit(1);
-		}
-		lbuffer.buffer = buffer;
-
-		retval = ioctl(fd, LKST_IOC_BUFFER_READ, &lbuffer);
-		if (retval){
-			fprintf(stderr, "ioctl() error: %s\n", strerror(errno));
-			printf("Skip reading this buffer!\n");
-			continue;
-		}
- 
-		if (file_set)
-			buffer_read_write_to_file(fd_o, &lbuffer);
-		else
-			buffer_read_display(&lbuffer);
-		
-		free(buffer);
-	}
-
-	if (file_set)
-		close(fd_o);
-
-	close(fd);
-
-	if (cnt == 0 && id != LKST_BUFFER_ID_VOID)
-		printf("Error: Specified buffer id=%d does not exist!\n", id);
-
-}
-
-inline void buffer_read_fast(lkst_buffer_id id, unsigned int num)
-{
-	int retval;
+	int retval, i;
 	int fd;
+	int flags;
 	struct lkst_log_buffer lbuffer;
 	struct lkst_log_record *buffer;
 
-	lbuffer.id = id;
-	lbuffer.read_size = sizeof(struct lkst_log_record) * num;
-
+	/*set log buffer*/
+	lbuffer.id = LKST_BUFFER_ID_VOID;
+	lbuffer.cpu = cpu;
+	if ( num != 0 ) 
+		lbuffer.read_size = sizeof(struct lkst_log_record) * num;
+	else 
+		lbuffer.read_size = sizeof(struct lkst_log_record) * 4096;
 	if (NULL == (buffer = (struct lkst_log_record *)malloc(lbuffer.read_size))) {
 		perror("malloc");
 		exit(1);
 	}
-
 	lbuffer.buffer = buffer;
 
+	/*device read*/
+	fd = open_dev();
+	flags = fcntl(fd, F_GETFL);
+	fcntl(fd, F_SETFL, flags|O_NONBLOCK);
+
+	if ( lkst_set_rmod(fd, cpu, &lbuffer) < 0){
+		fprintf(stderr, "lkst_set_rmod() error: %s\n", strerror(errno));
+		exit(1);
+	}
+	/*read loop*/
+	do {
+		if ( 0> (lbuffer.result_read_size = read(fd, lbuffer.buffer,
+							 lbuffer.read_size)) ){
+			fprintf(stderr, "read() error: %s\n", strerror(errno));
+			break;
+		}
+		if(fout <= 0)
+			buffer_display(&lbuffer);
+		else
+			buffer_write_to_file(fout, &lbuffer);
+	} while(num == 0 && lbuffer.read_size == lbuffer.result_read_size);
+	
+	free(buffer);
+
+	close(fd);
+
+}
+
+void buffer_read(int cpu, lkst_buffer_id id, unsigned int num, int fout)
+{
+	int retval, i;
+	int fd;
+	struct lkst_log_buffer lbuffer;
+	struct lkst_log_record *buffer;
+
 	fd = open_dev();
 
-	retval = ioctl(fd, LKST_IOC_BUFFER_READ, &lbuffer);	/* buff read */
-
-	if (retval) {
-		fprintf(stderr, "ioctl() error: %s\n", strerror(errno));
+	/*set log buffer */
+	lbuffer.id = id;
+	lbuffer.cpu = cpu;
+	if (id == LKST_BUFFER_ID_VOID ) {
+		lbuffer.read_size = LKST_STATIC_BUFFER_COUNT*sizeof(struct lkst_log_record);
+	} else if (num != 0){
+		lbuffer.read_size = sizeof(struct lkst_log_record) * num;
+	} else {
+		struct lkst_buffer_listparam lp;
+		struct lkst_buffer_listent listent[LKST_BUFFER_TBL_MAX]={0};
+		lp.cpu = cpu;
+		lp.listent_num = LKST_BUFFER_TBL_MAX;
+		lp.listent = listent;
+		if ( ioctl(fd, LKST_IOC_BUFFER_LIST, &lp) < 0){
+			fprintf(stderr, "lkst_buffer_list() error: %s\n", strerror(errno));
+			exit(1);
+		}
+		if(lp.listent_num < lp.buffer_num) lp.buffer_num = lp.listent_num;
+		for(i=0;i<lp.buffer_num && listent[i].id != id; i++);
+		if(i == lp.buffer_num) {
+			fprintf(stderr, "specified buffer not found: id = %d\n", id);
+			exit(1);
+		}
+		lbuffer.read_size = sizeof(struct lkst_log_record) * 
+			(listent[i].size/sizeof(struct lkst_event_record));
+	}
+	if (NULL == (buffer = (struct lkst_log_record *)malloc(lbuffer.read_size))) {
+		perror("malloc");
 		exit(1);
 	}
+	lbuffer.buffer = buffer;
 
-	buffer_read_display(&lbuffer);
+	/*device read*/
+	lkst_buffer_read(fd, &lbuffer);
+	if(fout <= 0)
+		buffer_display(&lbuffer);
+	else
+		buffer_write_to_file(fout, &lbuffer);
 
 	free(buffer);
 
 	close(fd);
 }
 
-inline void buffer_print_wonly(char*  program_name,    /* name of Execution file */
-				int   arg_cpu_id,
-				int   arg_filter_flag, /* 1=(-e option published) or 0=(not) */
-				int   arg_entry_num,
-				int   arg_sort_flag,
-				int   arg_csv_flag,
-				char* arg_inputfile,   /* name of reading file */
-				char* event_work)      /* The argument of "-e" option */
+//TODO : rewrite me!
+void buffer_print_wonly(char*  program_name,    /* name of Execution file */
+			int   arg_cpu_id,
+			int   arg_filter_flag, /* 1=(-e option published) or 0=(not) */
+			int   arg_entry_num,
+			int   arg_sort_flag,
+			int   arg_csv_flag,
+			char* arg_inputfile,   /* name of reading file */
+			char* event_work,      /* The argument of "-e" option */
+			int   verbose)
 {
 	FILE                             *fp;
 	int                              return_num;   /* return number of function */
@@ -443,6 +675,18 @@ inline void buffer_print_wonly(char*  program_name,    /* name of Execution file
 	struct lkst_log_record           **evr;
 	unsigned long                    *fdata;
 
+	if (verbose) { 
+		int i,j;
+		for (i=0;i<LKST_ETYPE_MAX;i++) {
+			for ( j= 0; j< 4;j++) {
+				if (( LKST_event_data[i].LKST_inf_args[j] == 0x00) ||
+				    (strcmp( LKST_event_data[i].LKST_inf_args[j], "") == 0)) {
+					LKST_event_data[i].LKST_inf_args[j] = UNDEF_STR;
+				}
+			}
+		}
+	}
+	
 	/* judgment of the inputted file name */
 	fp = fopen(arg_inputfile, LKST_LOG_FMT_FILE_OPEM_MODE);
 	if (fp==NULL) {
@@ -507,14 +751,14 @@ inline void buffer_print_wonly(char*  program_name,    /* name of Execution file
 	exit(0);
 }
 
-inline void buffer_print_randw(lkst_buffer_id  id, 
-				char*          program_name,
-				int            arg_cpu_id,
-				int            arg_filter_flag,
-				int	       arg_csv_flag,
-				int            arg_entry_num,
-				int            arg_sort_flag,
-				char           event_work[])
+void buffer_print_randw(lkst_buffer_id  id, 
+			char*          program_name,
+			int            arg_cpu_id,
+			int            arg_filter_flag,
+			int	       arg_csv_flag,
+			int            arg_entry_num,
+			int            arg_sort_flag,
+			char           event_work[])
 {
 	int                               return_num;
 	struct lkst_arg_command_analyze   pointcom;
@@ -533,17 +777,18 @@ inline void buffer_print_randw(lkst_buffer_id  id,
 		/* call function */
 		return_num = command_analyze(&(pointcom));
 		if (return_num < LKST_LOG_FMT_RET_CODE_TRUE) {
-			printf("%s: error in function-command_analyze\n\n", program_name);
+			fprintf(stderr,"%s: error in function-command_analyze(%d)\n\n", program_name, return_num);
 			exit(1);
 		}
 	}
 
 	/* get buffer data */
 	pointbuf.id      = id;
+	pointbuf.cpu	 = arg_cpu_id;
 	/* call function */
 	return_num = buffer_save(&(pointbuf));
 	if (return_num < LKST_LOG_FMT_RET_CODE_TRUE) {
-		printf("%s: error in function-command_analyze\n\n", program_name);
+		fprintf(stderr,"%s: error in function-buffer_save\n\n", program_name);
 		exit(1);
 	}
 
@@ -557,7 +802,7 @@ inline void buffer_print_randw(lkst_buffer_id  id,
 	/* call function */
 	return_num = make_pointerarray(&(pointarr));
 	if (return_num < LKST_LOG_FMT_RET_CODE_TRUE) {
-		printf("%s: error in function-command_analyze\n\n", program_name);
+		fprintf(stderr,"%s: error in function-make_pointerarray\n\n", program_name);
 		exit(1);
 	}
 
@@ -571,7 +816,7 @@ inline void buffer_print_randw(lkst_buffer_id  id,
 	/* call function */
 	return_num = sort_data(&(pointsor));
 	if (return_num < LKST_LOG_FMT_RET_CODE_TRUE) {
-		printf("%s: error in function-sort_data\n\n", program_name);
+		fprintf(stderr,"%s: error in function-sort_data\n\n", program_name);
 		exit(1);
 	}
 
@@ -584,7 +829,7 @@ inline void buffer_print_randw(lkst_buffer_id  id,
 	/* call function */
 	return_num = print_data(&(pointpri),arg_csv_flag);
 	if (return_num < LKST_LOG_FMT_RET_CODE_TRUE) {
-		printf("%s: error in function-print_data\n\n", program_name);
+		fprintf(stderr,"%s: error in function-print_data\n\n", program_name);
 		exit(1);
 	}
 
@@ -592,7 +837,6 @@ inline void buffer_print_randw(lkst_buffer_id  id,
 	free(evr);
 	free(fdata);
 	exit(0);
-
 }
 
 int command_analyze(struct lkst_arg_command_analyze *pointcom)
@@ -731,11 +975,12 @@ int buffer_save(struct lkst_arg_buffer_save *pointbuf)
 	fd = open_dev();
 
 	/* buffer list */
-	lp.listent_size = LKST_BUFFER_LISTENT_SIZE(LKST_BUFFER_TBL_MAX);
+	lp.cpu = pointbuf->cpu;
+	lp.listent_num = LKST_BUFFER_TBL_MAX;
 	lp.listent = listent;
 
-	if (retval = ioctl(fd, LKST_IOC_BUFFER_LIST, &lp)) {
-		fprintf(stderr, "ioctl() error: %s\n", strerror(errno));
+	if ((retval = ioctl(fd, LKST_IOC_BUFFER_LIST, &lp) < 0)) {
+		fprintf(stderr, "ioctl(LKST_IOC_BUFFER_LIST) error: %s\n", strerror(errno));
 		perror("buffer_list");
 		return LKST_LOG_FMT_RET_CODE_FALSE;
 	}
@@ -744,7 +989,7 @@ int buffer_save(struct lkst_arg_buffer_save *pointbuf)
 	}
 
 	/* measure size of a buffer */
-	for (i = 0; i < i_max; i++) {
+	for (i = 1; i < i_max; i++) {
 
 		if (i_max == 1) {
 			lbuffer.id = id;
@@ -807,8 +1052,8 @@ int buffer_save(struct lkst_arg_buffer_save *pointbuf)
 		printf("ioctl retval=%d result_read_size=%d\n", retval,
 			lbuffer.result_read_size);
 
-		if (retval) {
-			fprintf(stderr, "ioctl() error: %s\n", strerror(errno));
+		if (retval < 0) {
+			fprintf(stderr, "ioctl(LKST_IOC_BUFFER_READ) error: %s\n", strerror(errno));
 			printf("Skip reading this buffer!\n");
 			continue;
 		}
@@ -1107,17 +1352,17 @@ int print_data(struct lkst_arg_print_data *pointpri,int arg_csv_flag)
 
 		/* error about event_type_num */
 		if ((event_type_num > 0xffe) ||
-			(event_type_num == 0x00) ||
-			(LKST_event_data[event_type_num].LKST_event_name == NULL)) {
+		    (event_type_num == 0x00) ||
+		    (LKST_event_data[event_type_num].LKST_event_name == NULL)) {
 			continue;
 		}
 
 		/* display event_type */
-		for (j=0;j<(signed long)strlen(event_Name_str);j++) {
-			strncpy(&event_Name_str[j], "\0", 1);
-		}
-		strcat(event_Name_str, LKST_event_data[event_type_num].LKST_event_name);
-
+		strncpy(event_Name_str, 
+			LKST_event_data[event_type_num].LKST_event_name,
+			LKST_LOG_FMT_SORT_BUF_SIZE-1);
+		event_Name_str[LKST_LOG_FMT_SORT_BUF_SIZE-1]='\0';
+		
 		if (strcmp( event_Name_str, "" )==0) {
 			/* event_data undefined ? */
 			if ( arg_csv_flag )
@@ -1229,14 +1474,14 @@ void clock_viewer(unsigned long time_sec, unsigned long time_usec,int arg_csv_fo
 {
 	struct tm *newtime;
 	time_t    ltime;
-	char      monthName[12][4]  ={"Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"};
-	char      weekdayName[7][4] ={"Sun","Mon","Tue","Wed","Thu","Fri","Sat"};
+	char      monthName[12][4]={"Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"};
+	char      weekdayName[7][4]={"Sun","Mon","Tue","Wed","Thu","Fri","Sat"};
 
 	ltime = (time_t)time_sec;
 	newtime = localtime( &ltime );
 	if ( arg_csv_form )
 	{
-	printf( "%s,%s,%02d,%02d:%02d:%02d.%lu,%04d,",
+	printf( "%s,%s,%02d,%02d:%02d:%02d.%09lu,%04d,",
 		weekdayName[newtime->tm_wday],
 		monthName[newtime->tm_mon],
 		newtime->tm_mday,
@@ -1248,7 +1493,7 @@ void clock_viewer(unsigned long time_sec, unsigned long time_usec,int arg_csv_fo
 	}
 	else
 	{
-	printf( "\ttime=%s %s %02d %02d:%02d:%02d.%lu %04d\n",
+	printf( "\ttime=%s %s %02d %02d:%02d:%02d.%09lu %04d\n",
 		weekdayName[newtime->tm_wday],
 		monthName[newtime->tm_mon],
 		newtime->tm_mday,
@@ -1260,27 +1505,3 @@ void clock_viewer(unsigned long time_sec, unsigned long time_usec,int arg_csv_fo
 	}
 }
 
-#if DEBUG
-inline void buffer_seek(lkst_buffer_id id, int offset)
-{
-	int retval;
-	int fd;
-	struct lkst_buffer_seekparam bs;
-
-	fd = open_dev();
-
-	bs.id = id;
-	bs.offset = offset;
-
-	retval = ioctl(fd, LKST_IOC_BUFFER_SEEK, &bs);	/* buffer_seek */
-
-	if (retval) {
-		fprintf(stderr, "ioctl() error: %s\n", strerror(errno));
-		exit(1);
-	}
-
-	printf("Modify readp of buffer(id=%d) to %d\n", id, bs.result_readp);
-
-	close(fd);
-}
-#endif /* DEBUG */
