@@ -1,9 +1,6 @@
-/* $Id: lkst.c,v 1.4 2002/06/28 11:27:53 indoh Exp $ */
-/* command/lkst.c */
-
 /************************************************************
  *
- * COPYRIGHT (C) HITACHI,LTD. 2002 ALL RIGHTS RESERVED.
+ * COPYRIGHT (C) HITACHI,LTD. 2002-2003 ALL RIGHTS RESERVED.
  * WRITTEN BY HITACHI SYSTEMS DEVELOPMENT LABORATORY,
  *            HITACHI CENTRAL RESEARCH LABORATORY.
  *
@@ -12,9 +9,12 @@
  ************************************************************/
 
 #include "command.h"
+#include "buffer.h"
+#include "maskset.h"
+
 
 #define NAME	"lkst"
-#define VERSION "1.1"
+#define VERSION "1.2"
 
 #define NR_CMD 8
 
@@ -73,7 +73,7 @@ void usage(void)
 	printf("status/stat\tOutput a current status.\n");
 	printf("start\t\tStart event tracing.\n");
 	printf("stop\t\tStop event tracing.\n");
-	printf("version/ver\tPrint version information.\n");
+	printf("version/ver\tPrint version of this command.\n");
 	printf("help\t\tPrint this message.\n\n");
 	printf("<SEE ALSO>\n");
 	printf("lkstm\t\tControls Masksets.\n");
@@ -100,7 +100,7 @@ void call_all(int *argc, char **argv)
 	lkst_status();
 
 	printf("<List of buffers>\n");
-	buffer_list();
+	classical_buffer_list();
 
 	printf("<List of masksets>\n");
 	maskset_list();
@@ -129,26 +129,31 @@ void lkst_status(void)
 {
 	int retval;
 	int fd;
-	int i;
+	int i,cpu;
 	struct lkst_status_param param;
 
 	fd = open_dev();
 
 	retval = ioctl(fd, LKST_IOC_TRC_STATUS, &param); /* trc_status */
 
-	if (retval) {
-		fprintf(stderr, "ioctl() error: %s\n", strerror(errno));
+	if (retval<0) {
+		fprintf(stderr, "ioctl(LKST_IOC_TRC_STATUS) error: %s\n", strerror(errno));
 		exit(1);
 	}
 
+	cpu=0;
+	for(i = 0; i < LKST_CPU_MAX; i++)
+		if (param.online_cpu & (1UL<<i)) cpu++;
+	printf("version of LKST         : %d.%d.%d\n", param.version.major,
+	       param.version.minor, param.version.revision);
+	printf("number of cpus          : %d\n", cpu);
 	printf("number of masksets      : %d\n", param.maskset_num);
 	printf("number of event-handlers: %d\n", param.evhandler_num);
-	printf("number of buffers       : %d\n", param.buffer_num);
-	printf("current_maskset_id      : %d\n", param.current_maskset_id);
+	printf("current maskset_id      : %d\n", param.current_maskset_id);
 	for(i = 0; i < LKST_CPU_MAX; i++)
-		if (param.current_buffer_id[i] != LKST_BUFFER_ID_VOID)
-			printf("current_buffer_id (cpu: %03d): %d\n", i,
-			       param.current_buffer_id[i]);
+		if (param.online_cpu & (1UL<<i))
+			printf("current writing buffer_id (cpu: %03d): %d \n",
+			       i, param.write_buffer_id[i]);
 
 	close(fd);
 }
@@ -162,8 +167,8 @@ void lkst_start(void)
 
 	retval = ioctl(fd, LKST_IOC_TRC_START); /* trc_start */
 
-	if (retval) {
-		fprintf(stderr, "ioctl() error: %s\n", strerror(errno));
+	if (retval<0) {
+		fprintf(stderr, "ioctl(LKST_IOC_TRC_START) error: %s\n", strerror(errno));
 		exit(1);
 	}
 
@@ -181,8 +186,8 @@ void lkst_stop(void)
 
 	retval = ioctl(fd, LKST_IOC_TRC_STOP); /* trc_stop */
 
-	if (retval) {
-		fprintf(stderr, "ioctl() error: %s\n", strerror(errno));
+	if (retval<0) {
+		fprintf(stderr, "ioctl(LKST_IOC_TRC_STOP) error: %s\n", strerror(errno));
 		exit(1);
 	}
 
