@@ -1,48 +1,43 @@
-/* $Id: lkstbuf.c,v 1.7 2002/06/28 11:27:53 indoh Exp $ */
-/* command/lkstbuf.c */
-
 /************************************************************
  *
- * COPYRIGHT (C) HITACHI,LTD. 2002 ALL RIGHTS RESERVED.
+ * COPYRIGHT (C) HITACHI,LTD. 2002-2003 ALL RIGHTS RESERVED.
  * WRITTEN BY HITACHI SYSTEMS DEVELOPMENT LABORATORY,
  *            HITACHI CENTRAL RESEARCH LABORATORY.
  *
  * Created by K.Hatasaki <keisukeh@crl.hitachi.co.jp>
+ * Updated by M.Hiramatsu <hiramatu@sdl.hitachi.co.jp>
  *
  ************************************************************/
 
+#include <stdlib.h>
 #include "command.h"
+#include "buffer.h"
 
 #define NAME	"lkstbuf"
-#define VERSION "1.2"
-
-#define NR_CMD  11
-#if DEBUG
-#define NR_CMD  12
-#endif /* DEBUG */
+#define VERSION "1.6"
 
 void usage(void);
 
 void call_usage(int *, char **);
 void call_version(int*, char**);
 void call_list(int*, char**);
+void call_link(int*, char**);
 void call_create(int*, char**);
 void call_del(int*, char**);
 void call_read(int*, char**);
 void call_shift(int*, char**);
-
-#if DEBUG
-void call_seek(int*, char**);
-#endif /* DEBUG */
+void call_jump(int*, char**);
 
 void call_print(int*, char**);
 int  nors_analyzer(char*);
 
+#define NR_CMD  14
+
 int main(int argc, char **argv)
 {
 	int i;
 
-	struct command cmd[] = 
+	struct command cmd[NR_CMD] = 
 	{
 		[0] = {"help", &call_usage},
 		[1] = {"ver", &call_version},
@@ -55,9 +50,9 @@ int main(int argc, char **argv)
 		[8] = {"read", &call_read},
 		[9] = {"shift", &call_shift},
 		[10] = {"print", &call_print},
-#if DEBUG
-		[11] = {"seek", &call_seek},
-#endif /* DEBUG */
+		[11] = {"link", &call_link},
+		[12] = {"ln", &call_link},
+		[13] = {"jump", &call_jump},
 	};
 
 	if (argc == 1) { /* no command */
@@ -84,23 +79,30 @@ void usage(void)
 	printf("shift\t\tChange currently selected buffer to next buffer.\n");
 	printf("\t\t<OPTION>\n");
 	printf("\t\t[-c cpu_id]\n");
+	printf("jump\t\tChange currently selected buffer to specified buffer.\n");
+	printf("\t\t<OPTION>\n");
+	printf("\t\t[-c cpu_id] -b buffer_id\n");
 	printf("create\t\tCreate a new kernel event buffer\n");
 	printf("\t\t<OPTION>\n");
-	printf("\t\t[-b buffer_id] [-c cpu_id] -s size\n");
+	printf("\t\t[-c cpu_id] -s size[K|M] [-b buffer_id] [-n next-buffer_id]\n");
 	printf("delete/del\tDelete a kernel event buffer.\n");
 	printf("\t\t<OPTION>\n");
-	printf("\t\t-b buffer_id\n");
+	printf("\t\t[-c cpu_id] -b buffer_id\n");
+	printf("link/ln\t\tSet the next buffer for the shift operation.\n");
+	printf("\t\t<OPTION>\n");
+	printf("\t\t[-c cpu_id] -b buffer_id [-n next_buffer_id]\n");	
 	printf("list/ls\t\tOutput a list of all kernel event buffers.\n");
+	printf("\t\t<OPTION>\n");
+	printf("\t\t[-c cpu_id] [-v]\n");
 	printf("read\t\tOutput a content of kernel event buffer(s).\n");
 	printf("\t\t<OPTION>\n");
-	printf("\t\t[-b buffer_id] [-f output_file] [-n number]\n");
+	printf("\t\t[-c cpu_id] [-b buffer_id] [-f output_file] [-n number]\n");
+	printf("\t\t-S [-f output_file] [-n number]\n");
 	printf("print\t\tOutput a content of kernel event buffer(s) infomation.\n");
 	printf("\t\t<OPTION>\n");
-	printf("\t\t[-C]\n");
-	printf("\t\t[-b buffer_id]\n");
-	printf("\t\t[-f output_file] [-c cpu_id] [-e 'event_name,!eventname']\n");
-	printf("\t\t[-n number] [-r] [-h]\n");
-	printf("version/ver\tPrint version information.\n");
+	printf("\t\t-f log_file [-c cpu_id] [-e 'event_name,!eventname']\n");
+	printf("\t\t\t[-n number] [-r] [-C] [-V] [-h]\n");
+	printf("version/ver\tPrint version of this command.\n");
 	printf("help\t\tPrint this message.\n");
 	exit(0);
 }
@@ -117,29 +119,87 @@ void call_version(int *argc, char **argv)
 
 void call_list(int *argc, char **argv)
 {
-	buffer_list();
+	int c,cpu=-1,v=0;
+	while((c = getopt(*argc, argv, "c:v")) != EOF) {
+		switch(c) {
+		case 'c' :
+			cpu = strtol(optarg,NULL,0);
+			break;
+		case 'v' :
+			v=1;
+			break;
+		default :
+			usage();
+		}
+	}
+	if(v)
+		buffer_list(cpu,0);
+	else
+		classical_buffer_list();
+}
+
+void call_link(int *argc, char **argv)
+{
+	int c;
+	int size_set = 0;
+	int cpu = -1;
+	lkst_buffer_id id = LKST_BUFFER_ID_VOID;
+	lkst_buffer_id next = LKST_BUFFER_ID_VOID;
+
+	while((c = getopt(*argc, argv, "b:c:n:")) != EOF) {
+		switch(c) {
+		case 'b' :
+			id = strtol(optarg,NULL,0);
+			break;
+		case 'n' :
+			next = strtol(optarg,NULL,0);
+			break;
+		case 'c' :
+			cpu = strtol(optarg,NULL,0);
+			break;
+		default :
+			usage();
+		}
+	}
+
+	if (id==LKST_BUFFER_ID_VOID)
+		needopt();
+
+	buffer_link(cpu, id, next);
 }
 
 void call_create(int *argc, char **argv)
 {
 	int c;
-	int i;
 	int size_set = 0;
 	int cpu = -1;
 	lkst_buffer_id id = LKST_BUFFER_ID_VOID;
+	lkst_buffer_id nxid = LKST_BUFFER_ID_VOID;
 	size_t size;
+	char * tmp;
 
-	while((c = getopt(*argc, argv, "b:c:s:")) != EOF) {
+	while((c = getopt(*argc, argv, "b:c:s:n:")) != EOF) {
 		switch(c) {
 		case 'b' :
-			id = atoi(optarg);
+			id = strtol(optarg,NULL,0);
 			break;
 		case 'c' :
-			cpu = atoi(optarg);
+			cpu = strtol(optarg,NULL,0);
 			break;
 		case 's' :
 			size_set = 1;
-			size = atoi(optarg);
+			size = strtol(optarg,&tmp,0);
+			if(tmp[0] == 'M' ) {
+				tmp ++;
+				size *= 1024*1024;
+			}else if(tmp[0] == 'K') {
+				tmp ++;
+				size *= 1024;
+			}
+			if (tmp[0]!='\0') usage();
+			break;
+		case 'n':
+			nxid = strtol(optarg,NULL,0);
 			break;
 		default :
 			usage();
@@ -149,89 +209,146 @@ void call_create(int *argc, char **argv)
 	if (!size_set)
 		needopt();
 
-	buffer_create(cpu, id, size);
+	buffer_create(cpu, id, size, nxid);
 }
 
 void call_del(int *argc, char **argv)
 {
 	int c;
-	lkst_buffer_id id;
+	lkst_buffer_id id=LKST_BUFFER_ID_VOID;
+	int cpu=-1;
 
-	c = getopt(*argc, argv, "b:");
-
-	switch(c) {
-	case 'b' :
-		id = atoi(optarg);
-		break;
-	case -1:
-		needopt();
-		break;
-	default :
-		usage();
-		break;
+	while((c = getopt(*argc, argv, "b:c:")) != EOF) {
+		switch(c) {
+		case 'b' :
+			id = strtol(optarg,NULL,0);
+			break;
+		case 'c' :
+			cpu = strtol(optarg,NULL,0);
+			break;
+		default :
+			usage();
+		}
 	}
+	if(id == LKST_BUFFER_ID_VOID)
+		needopt();
 
-	buffer_delete(id);
+	buffer_delete(cpu, id);
 }
 
 void call_read(int *argc, char **argv)
 {
 	int c;
-	int i;
-	int num_set = 0;
+	int fd = -1;
+	int cpu = -1, online = 0;
 	int file_set = 0;
+	int static_buffer = 0;
 	lkst_buffer_id id = LKST_BUFFER_ID_VOID;
 	char file_name[255];
-	unsigned int num;
+	unsigned int num = 0;
 
-	while((c = getopt(*argc, argv, "b:n:f:")) != EOF) {
+	while((c = getopt(*argc, argv, "c:b:n:f:S")) != EOF) {
 		switch(c) {
+		case 'c':
+			if(static_buffer)usage();
+			cpu = strtol(optarg,NULL,0);
+			break;
 		case 'b' :
-			id = atoi(optarg);
+			if(static_buffer)usage();
+			id = strtol(optarg,NULL,0);
 			break;
 		case 'n' :
-			num_set = 1;
-			num = atoi(optarg);
+			num = strtol(optarg,NULL,0);
 			break;
 		case 'f' :
 			file_set = 1;
 			strncpy(file_name, optarg, 255);
 			break;
+		case 'S':
+			if(cpu!=-1||id!=LKST_BUFFER_ID_VOID)usage();
+			static_buffer = 1;
+			break;
 		default :
 			usage();
 		}
 	}
 
-	if (num_set && id != LKST_BUFFER_ID_VOID && !file_set)
-		buffer_read_fast(id, num);
-	else 
-		buffer_read(id, num, file_name, num_set, file_set);
+	if(cpu != -1 && static_buffer!=0)
+		needopt();
+
+	if (file_set) {
+		if (0 == strncmp("-", file_name, 2))
+			fd = 1;/* stdout */
+		else if ((fd = open(file_name, O_WRONLY|O_CREAT|O_TRUNC,0644)) < 0){
+			perror("output");
+			exit(1);
+		}
+	}
+
+	if ( static_buffer ) {
+		buffer_read(-1, LKST_BUFFER_ID_VOID, num, fd);
+		return ;
+	}
+	
+	if(cpu == -1 ) { // then, fork per processor.
+		struct lkst_status_param sparam;
+		if ( lkst_get_status(&sparam) < 0){
+			perror("get status");
+			exit(1);
+		}
+		online = sparam.online_cpu;
+	}else 
+		online = 1UL << cpu;
+
+	for_each_online_cpus(cpu, online){
+		if( id == LKST_BUFFER_ID_VOID)
+			buffer_read_all(cpu, num, fd);
+		else 
+			buffer_read(cpu, id, num, fd);
+	}
 }
 
 void call_shift(int *argc, char **argv)
 {
 	int c;
-	int cpu;
-	int all = 0;
+	int cpu = -1;
 
-	c = getopt(*argc, argv, "c:");
-
-	switch(c) {
-	case 'c' :
-		cpu = atoi(optarg);
-		break;
-	case -1:
-		all = 1;
-		break;
-	default :
-		usage();
-		break;
+	if ( (c = getopt(*argc, argv, "c:")) != EOF) {
+		switch(c) {
+		case 'c' :
+			cpu = strtol(optarg,NULL,0);
+			break;
+		default :
+			usage();
+			break;
+		}
 	}
 
-	if (all)
-		buffer_shift_all();
-	else
-		buffer_shift(cpu);
+	buffer_shift(cpu);
+}
+
+void call_jump(int *argc, char **argv)
+{
+	int c;
+	lkst_buffer_id dst=LKST_BUFFER_ID_VOID;
+	int cpu=-1;
+
+	while((c = getopt(*argc, argv, "b:c:")) != EOF) {
+		switch(c) {
+		case 'b' :
+			dst = strtol(optarg,NULL,0);
+			break;
+		case 'c' :
+			cpu = strtol(optarg,NULL,0);
+			break;
+		default :
+			usage();
+		}
+	}
+	if(dst == LKST_BUFFER_ID_VOID)
+		needopt();
+
+	buffer_jump(cpu, dst);
 }
 
 void call_print(int *argc, char **argv)
@@ -244,6 +361,7 @@ void call_print(int *argc, char **argv)
 	int  arg_sort_flag   = 0;
 	int  help_flag       = 0;
 	int  csv_form_flag   = 0;
+	int  verbose = 0;
 	char* program_name;
 	char arg_inputfile[255];
 	char event_work[255];
@@ -251,11 +369,11 @@ void call_print(int *argc, char **argv)
 
 	program_name = argv[0];
 
-	while ((c = getopt(*argc, argv, "b:f:c:e:n:rhC")) != EOF) {
+	while ((c = getopt(*argc, argv, "b:f:c:e:n:rhCV")) != EOF) {
 		switch (c) {
 		case 'b' :
 			if (nors_analyzer(optarg)) {
-				id = atoi(optarg);
+				id = strtol(optarg,NULL,0);
 			} else {
 				usage();
 			}
@@ -266,7 +384,7 @@ void call_print(int *argc, char **argv)
 			break;
 		case 'c' :
 			if (nors_analyzer(optarg)) {
-				arg_cpu_id = atoi(optarg);
+				arg_cpu_id = strtol(optarg,NULL,0);
 			} else {
 				usage();
 			}
@@ -277,7 +395,7 @@ void call_print(int *argc, char **argv)
 			break;
 		case 'n' :
 			if (nors_analyzer(optarg)) {
-				arg_entry_num = atoi(optarg);
+				arg_entry_num = strtol(optarg,NULL,0);
 			} else {
 				usage();
 			}
@@ -291,6 +409,9 @@ void call_print(int *argc, char **argv)
 		case 'C':
 			csv_form_flag = 1;
 			break;
+		case 'V':
+			verbose ++;
+			break;
 		default :
 			usage();
 			break;
@@ -303,10 +424,11 @@ void call_print(int *argc, char **argv)
 	}
 	if (file_set) {
 		buffer_print_wonly(program_name, arg_cpu_id, arg_filter_flag,
-				   arg_entry_num, arg_sort_flag, csv_form_flag,arg_inputfile, event_work);
-	} else {
-		buffer_print_randw(id, program_name, arg_cpu_id, arg_filter_flag, csv_form_flag,arg_entry_num, arg_sort_flag, event_work);
-	}
+				   arg_entry_num, arg_sort_flag, 
+				   csv_form_flag,arg_inputfile, 
+				   event_work, verbose);
+	}else
+		needopt();
 }
 
 int nors_analyzer(char* str)
@@ -319,35 +441,3 @@ int nors_analyzer(char* str)
 	}
 	return 1;
 }
-
-#if DEBUG
-void call_seek(int *argc, char **argv)
-{
-	int c;
-	int i;
-	int id_set;
-	int offset_set;
-	lkst_buffer_id id;
-	size_t offset;
-
-	while((c = getopt(*argc, argv, "b:o:")) != EOF) {
-		switch(c) {
-		case 'b' :
-			id_set = 1;
-			id = atoi(optarg);
-			break;
-		case 'o' :
-			offset_set = 1;
-			offset = atoi(optarg);
-			break;
-		default :
-			usage();
-		}
-	}
-
-	if (!id_set || !offset_set)
-		needopt();
-
-	buffer_seek(id, offset);
-}
-#endif /* DEBUG */
