@@ -37,7 +37,8 @@ struct timespec base_time;
 int buffer_id;
 int cmos_flag;
 FILE* fp_output;
-static struct lkst_header *lkst;
+static struct lkst_header *lkst=NULL;
+static struct lkst_header_percpu *lkst_cpu=NULL;
 
 VALUE_S* isDigitStr(VALUE_S *vp)
 {
@@ -78,11 +79,24 @@ VALUE_S* lkst_getoffset_lkst_h_cmos_flag()
 	return sial_makebtype(offset);
 }
 
-VALUE_S* lkst_getoffset_lkst_h_bufferp(VALUE_S* vi)
+VALUE_S* lkst_getoffset_lkst_cpu_h()
+{
+	int offset = (char*)(&lkst_cpu[1]) - (char*)(&lkst_cpu[0]);
+	return sial_makebtype(offset);
+}
+
+VALUE_S* lkst_getoffset_lkst_cpu_h_bufferp(VALUE_S* vi)
 {
 	int offset;
 	int i=sial_getval(vi);
-	offset = offsetof( struct lkst_header, buffer_table[i].bufferp );
+	offset = offsetof( struct lkst_header_percpu, buffer_table[i].ptr );
+	return sial_makebtype(offset);
+}
+
+VALUE_S* lkst_getoffset_lkst_h_sbuf_body()
+{
+	int offset;
+	offset = offsetof( struct lkst_header, static_buffer.body );
 	return sial_makebtype(offset);
 }
 
@@ -114,6 +128,16 @@ VALUE_S* lkst_getvalue_buffer_id_void()
 	return sial_makebtype(LKST_BUFFER_ID_VOID);
 }
 
+VALUE_S* lkst_getvalue_cpu_max()
+{
+	return sial_makebtype(LKST_CPU_MAX);
+}
+
+VALUE_S* lkst_getvalue_sbuf_size()
+{
+	return sial_makebtype(LKST_STATIC_BUFFER_COUNT);
+}
+
 VALUE_S* lkst_getvalue_evrec_size()
 {
 	return sial_makebtype(sizeof(struct lkst_event_record));
@@ -222,13 +246,25 @@ VALUE_S* lkst_check_and_init(VALUE_S* lkst_addr)
 	char   magic_num[5];
 
 	lkst = (struct lkst_header*)(long)sial_getval(lkst_addr);
-	sial_getmem( (ull)(long)(lkst->eyecacher), &(magic_num[0]), sizeof(lkst->eyecacher) );
+	sial_getmem( (ull)(long)(lkst->eyecatcher), &(magic_num[0]), sizeof(lkst->eyecatcher) );
 	if( strncmp( magic_num, "LKST", 4) != 0 ) { /* check the magic number 'L','K','S','T' */
 		return sial_makebtype(1);
 	}
 	return sial_makebtype(0);
 }
 
+VALUE_S* lkst_check_and_init_cpu(VALUE_S* lkst_cpu_addr)
+{
+	char   magic_num[5];
+
+	lkst_cpu = (struct lkst_header_percpu*)(long)sial_getval(lkst_cpu_addr);
+	sial_getmem( (ull)(long)(lkst_cpu->eyecatcher), &(magic_num[0]), sizeof(lkst_cpu->eyecatcher) );
+	if( strncmp( magic_num, "LKCP", 4) != 0 ) { /* check the magic number 'L','K','S','T' */
+		return sial_makebtype(1);
+	}
+	return sial_makebtype(0);
+}
+
 VALUE_S* lkst_open_write_file(VALUE_S* vfname)
 {
 	char* fname = (char*)(long)sial_getval(vfname);
@@ -285,10 +321,14 @@ BT_SPEC_TABLE = {
 	{ "int lkst_getoffset_lkst_h_tsc()",              (void*)lkst_getoffset_lkst_h_tsc},
 	{ "int lkst_getoffset_lkst_h_cpu_freq()",         (void*)lkst_getoffset_lkst_h_cpu_freq},
 	{ "int lkst_getoffset_lkst_h_cmos_flag()",        (void*)lkst_getoffset_lkst_h_cmos_flag},
-	{ "int lkst_getoffset_lkst_h_bufferp(int i)",     (void*)lkst_getoffset_lkst_h_bufferp},
+	{ "int lkst_getoffset_lkst_cpu_h()",		  (void*)lkst_getoffset_lkst_cpu_h},
+	{ "int lkst_getoffset_lkst_cpu_h_bufferp(int i)", (void*)lkst_getoffset_lkst_cpu_h_bufferp},
+	{ "int lkst_getoffset_lkst_h_sbuf_body()",        (void*)lkst_getoffset_lkst_h_sbuf_body},
 	{ "int lkst_getoffset_evbuf_id()",                (void*)lkst_getoffset_evbuf_id},
 	{ "int lkst_getoffset_evbuf_size()",              (void*)lkst_getoffset_evbuf_size},
 	{ "int lkst_getoffset_evbuf_body()",              (void*)lkst_getoffset_evbuf_body},
+	{ "int lkst_getvalue_sbuf_size()",                (void*)lkst_getvalue_sbuf_size},
+	{ "int lkst_getvalue_cpu_max()",                  (void*)lkst_getvalue_cpu_max},
 	{ "int lkst_getvalue_buffer_id_max()",            (void*)lkst_getvalue_buffer_id_max},
 	{ "int lkst_getvalue_evrec_size()",               (void*)lkst_getvalue_evrec_size},
 	{ "int lkst_getvalue_buffer_id_void()",           (void*)lkst_getvalue_buffer_id_void},
@@ -302,12 +342,13 @@ BT_SPEC_TABLE = {
 	{ "int lkst_setdata_buffer_id(int data)",         (void*)lkst_setdata_buffer_id},
 	{ "int lkst_open_write_file(string vfname)",      (void*)lkst_open_write_file},
 	{ "int lkst_close_write_file()",                  (void*)lkst_close_write_file},
-	{ "int lkst_alloc_buffer(int size)", (void*)lkst_alloc_buffer},
+	{ "int lkst_alloc_buffer(int size)", 		  (void*)lkst_alloc_buffer},
 	{ "int lkst_copy_buffer_pointer(int ptr, int len)", (void*)lkst_copy_buffer_pointer},
-	{ "int lkst_copy_buffer(int idx, int data)", (void*)lkst_copy_buffer},
-	{ "int lkst_write_buffer_file()", (void*)lkst_write_buffer_file},
-	{ "int lkst_release_buffer()", (void*)lkst_release_buffer},
-	{ "int lkst_check_and_init(int lkst_addr)", (void*)lkst_check_and_init},
+	{ "int lkst_copy_buffer(int idx, int data)",	  (void*)lkst_copy_buffer},
+	{ "int lkst_write_buffer_file()",		  (void*)lkst_write_buffer_file},
+	{ "int lkst_release_buffer()",			  (void*)lkst_release_buffer},
+	{ "int lkst_check_and_init(int lkst_addr)",	  (void*)lkst_check_and_init},
+	{ "int lkst_check_and_init_cpu(int lkst_cpu_addr)", (void*)lkst_check_and_init_cpu},
 	{ 0, 0}
 };
 
@@ -358,7 +399,7 @@ void lkst_write_buffer(struct lkst_event_record* pevt_rec, int nrec)
 		plog_rec[j].posix.log_pgrp         = -1;
 		plog_rec[j].posix.log_flags        = 0;
 		plog_rec[j].posix.log_thread       = -1;
-		plog_rec[j].posix.log_processor    = 0;
+		plog_rec[j].posix.log_processor    = pevt_rec[j]._reserved0;
 		plog_rec[j].log_arg1 = pevt_rec[j].log_arg1;
 		plog_rec[j].log_arg2 = pevt_rec[j].log_arg2;
 		plog_rec[j].log_arg3 = pevt_rec[j].log_arg3;
